% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colby_constructors.R
\name{split_cols_by_multivar}
\alias{split_cols_by_multivar}
\title{Associate Multiple Variables with Columns}
\usage{
split_cols_by_multivar(
  lyt,
  vars,
  varlabels = vars,
  varnames = NULL,
  nested = TRUE
)
}
\arguments{
\item{lyt}{layout object pre-data used for tabulation}

\item{vars}{character vector. Multiple variable names.}

\item{varlabels}{character vector. Labels for \code{vars}}

\item{varnames}{character vector. Names for \code{vars} which will appear in pathing. When \code{vars} are all unique this will be the variable names. If not, these will be variable names with suffixes as necessary to enforce uniqueness.}

\item{nested}{boolean, Add this as a new top-level split  (defining a new subtable directly under root). Defaults to \code{FALSE}}
}
\value{
A \code{PreDataTableLayouts} object suitable for passing to further layouting functions, and to \code{build_table}.
}
\description{
In some cases, the variable to be ultimately analyzed is most naturally defined on a column, not a row basis. When we
need columns to reflect different variables entirely, rather than different levels of a single variable, we use
\code{split_cols_by_multivar}
}
\examples{

library(dplyr)
ANL <- DM \%>\% mutate(value = rnorm(n()), pctdiff = runif(n()))

## toy example where we take the mean of the first variable and the
## count of >.5 for the second.
colfuns <- list(function(x) in_rows(mean = mean(x), .formats = "xx.x"),
                function(x) in_rows("# x > 5" = sum(x > .5), .formats = "xx"))

l <- basic_table() \%>\%
    split_cols_by("ARM") \%>\%
    split_cols_by_multivar(c("value", "pctdiff")) \%>\%
    split_rows_by("RACE", split_label = "ethnicity", split_fun = drop_split_levels) \%>\%
    summarize_row_groups() \%>\%
    analyze_colvars(afun = colfuns)

l

build_table(l, ANL)

}
\seealso{
\code{\link{analyze_colvars}}
}
\author{
Gabriel Becker
}
